(function (factory) {
    if (typeof module === 'object' && typeof module.exports === 'object') {
        var v = factory(require, exports); if (v !== undefined) module.exports = v;
    }
    else if (typeof define === 'function' && define.amd) {
        define(["require", "exports", '../../util/util'], factory);
    }
})(function (require, exports) {
    "use strict";
    var util_1 = require('../../util/util');
    var ImgLoader = (function () {
        function ImgLoader() {
            this.callbacks = [];
            this.ids = 0;
        }
        ImgLoader.prototype.load = function (src, cache, callback) {
            if (src) {
                callback.id = this.ids++;
                this.callbacks.push(callback);
                this.worker().postMessage(JSON.stringify({
                    id: callback.id,
                    src: src,
                    cache: cache
                }));
            }
        };
        ImgLoader.prototype.cancelLoad = function (callback) {
            util_1.removeArrayItem(this.callbacks, callback);
        };
        ImgLoader.prototype.abort = function (src) {
            if (src) {
                this.worker().postMessage(JSON.stringify({
                    src: src,
                    type: 'abort'
                }));
            }
        };
        ImgLoader.prototype.worker = function () {
            var _this = this;
            if (!this.wkr) {
                var workerBlob = new Blob([INLINE_WORKER]);
                var blobURL = window.URL.createObjectURL(workerBlob);
                this.wkr = new Worker(blobURL);
                this.wkr.onmessage = function (ev) {
                    var msg = JSON.parse(ev.data);
                    var callback = _this.callbacks.find(function (cb) { return cb.id === msg.id; });
                    if (callback) {
                        callback(msg);
                        util_1.removeArrayItem(_this.callbacks, callback);
                    }
                };
                this.wkr.onerror = function (ev) {
                    console.error("ImgLoader, worker " + ev.type + " " + (ev.message ? ev.message : ''));
                    _this.callbacks.length = 0;
                    _this.wkr.terminate();
                    _this.wkr = null;
                };
            }
            return this.wkr;
        };
        return ImgLoader;
    }());
    exports.ImgLoader = ImgLoader;
    var INLINE_WORKER = "/** minify-start **/\n\n(function(){\n\n  var imgs = [];\n  var encodings = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';\n  var cacheLimit = 1381855 * 10;\n\n  onmessage = function(msg) {\n    var msgData = JSON.parse(msg.data);\n    var id = msgData.id;\n    var src = msgData.src;\n    var imgData;\n\n    for (var i = 0; i < imgs.length; i++) {\n      if (imgs[i].s === src) {\n        imgData = imgs[i];\n        break;\n      }\n    }\n\n    if (msgData.type === 'abort') {\n      if (imgData && imgData.x) {\n        imgData.x.abort();\n        imgData.x = null;\n      }\n\n    } else if (msgData.cache && imgData && imgData.d) {\n      postMessage(JSON.stringify({\n        id: id,\n        src: src,\n        status: 200,\n        data: imgData.d,\n        len: imgData.l\n      }));\n\n    } else {\n      if (imgData && imgData.x && imgData.x.readyState !== 4) {\n        imgData.x.addEventListener('load', function(ev) {\n          onXhrLoad(id, src, imgData, ev);\n        });\n        imgData.x.addEventListener('error', function(e) {\n          onXhrError(id, src, imgData, e);\n        });\n        return;\n      }\n\n      if (!imgData) {\n        imgData = { s: src, c: msgData.cache };\n        imgs.push(imgData);\n      }\n\n      imgData.x = new XMLHttpRequest();\n      imgData.x.open('GET', src, true);\n      imgData.x.responseType = 'arraybuffer';\n      imgData.x.addEventListener('load', function(ev) {\n        onXhrLoad(id, src, imgData, ev);\n      });\n      imgData.x.addEventListener('error', function(e) {\n        onXhrError(id, src, imgData, e);\n      });\n      imgData.x.send();\n    }\n\n  };\n\n  function onXhrLoad(id, src, imgData, ev) {\n    var rsp = {\n      id: id,\n      src: src,\n      status: ev.target.status,\n      data: null,\n      len: 0\n    };\n\n    if (ev.target.status === 200) {\n      setData(rsp, ev.target.getResponseHeader('Content-Type'), ev.target.response);\n      rsp.len = rsp.data.length;\n    }\n\n    postMessage(JSON.stringify(rsp));\n\n    if (imgData.x.status === 200 && imgData.c) {\n      imgData.d = rsp.data;\n      imgData.l = rsp.len;\n\n      var cacheSize = 0;\n      for (var i = imgs.length - 1; i >= 0; i--) {\n        cacheSize += imgs[i].l;\n        if (cacheSize > cacheLimit) {\n          imgs.splice(i, 1);\n        }\n      }\n    }\n  };\n\n  function onXhrError(id, src, imgData, e) {\n    postMessage(JSON.stringify({\n      id: id,\n      src: src,\n      status: 510,\n      msg: e.message + '' + e.stack\n    }));\n    imgData.x = null;\n  };\n\n\n  function setData(rsp, contentType, arrayBuffer) {\n    rsp.data = 'data:' + contentType + ';base64,';\n\n    var bytes = new Uint8Array(arrayBuffer);\n    var byteLength = bytes.byteLength;\n    var byteRemainder = byteLength % 3;\n    var mainLength = byteLength - byteRemainder;\n    var i, a, b, c, d, chunk;\n\n    for (i = 0; i < mainLength; i = i + 3) {\n      chunk = (bytes[i] << 16) | (bytes[i + 1] << 8) | bytes[i + 2];\n      a = (chunk & 16515072) >> 18;\n      b = (chunk & 258048) >> 12;\n      c = (chunk & 4032) >> 6;\n      d = chunk & 63;\n      rsp.data += encodings[a] + encodings[b] + encodings[c] + encodings[d];\n    }\n\n    if (byteRemainder === 1) {\n      chunk = bytes[mainLength];\n      a = (chunk & 252) >> 2;\n      b = (chunk & 3) << 4;\n      rsp.data += encodings[a] + encodings[b] + '==';\n\n    } else if (byteRemainder === 2) {\n      chunk = (bytes[mainLength] << 8) | bytes[mainLength + 1];\n      a = (chunk & 64512) >> 10;\n      b = (chunk & 1008) >> 4;\n      c = (chunk & 15) << 2;\n      rsp.data += encodings[a] + encodings[b] + encodings[c] + '=';\n    }\n  }\n\n})();\n\n/** minify-end **/";
});
//# sourceMappingURL=img-loader.js.map